@@ -57,6 +57,7 @@ SRC_DST_SAME_MSG = "ERR source and destination objects are the same"
 NO_KEY_MSG = "ERR no such key"
 INDEX_ERROR_MSG = "ERR index out of range"
 ZADD_NX_XX_ERROR_MSG = "ERR ZADD allows either 'nx' or 'xx', not both"
+ZADD_INCR_LEN_ERROR_MSG = "ERR INCR option supports a single increment-element pair"
 ZUNIONSTORE_KEYS_MSG = "ERR at least 1 input key is needed for ZUNIONSTORE/ZINTERSTORE"
 WRONG_ARGS_MSG = "ERR wrong number of arguments for '{}' command"
 UNKNOWN_COMMAND_MSG = "ERR unknown command '{}'"
@@ -1047,7 +1048,8 @@ class FakeSocket:
         if not key or key.key in self._server.dbs[db]:
             return 0
         # TODO: what is the interaction with expiry?
-        self._server.dbs[db][key.key] = self._server.dbs[self._db_num].pop(key.key)
+        self._server.dbs[db][key.key] = self._server.dbs[self._db_num][key.key]
+        key.value = None   # Causes deletion
         return 1
 
     @command(())
@@ -1408,6 +1410,7 @@ class FakeSocket:
         px = None
         xx = False
         nx = False
+        keepttl = False
         while i < len(args):
             if casematch(args[i], b'nx'):
                 nx = True
@@ -1425,16 +1428,22 @@ class FakeSocket:
                 if px <= 0:
                     raise SimpleError(INVALID_EXPIRE_MSG.format('set'))
                 i += 2
+            elif casematch(args[i], b'keepttl'):
+                keepttl = True
+                i += 1
             else:
                 raise SimpleError(SYNTAX_ERROR_MSG)
-        if (xx and nx) or (px is not None and ex is not None):
+        if (xx and nx) or ((px is not None) + (ex is not None) + keepttl > 1):
             raise SimpleError(SYNTAX_ERROR_MSG)
 
         if nx and key:
             return None
         if xx and not key:
             return None
-        key.value = value
+        if not keepttl:
+            key.value = value
+        else:
+            key.update(value)
         if ex is not None:
             key.expireat = self._db.time + ex
         if px is not None:
@@ -1949,13 +1958,13 @@ class FakeSocket:
 
     @command((Key(ZSet), bytes, bytes), (bytes,))
     def zadd(self, key, *args):
-        # TODO: handle INCR
         zset = key.value
 
         i = 0
         ch = False
         nx = False
         xx = False
+        incr = False
         while i < len(args):
             if casematch(args[i], b'ch'):
                 ch = True
@@ -1966,6 +1975,9 @@ class FakeSocket:
             elif casematch(args[i], b'xx'):
                 xx = True
                 i += 1
+            elif casematch(args[i], b'incr'):
+                incr = True
+                i += 1
             else:
                 # First argument not matching flags indicates the start of
                 # score pairs.
@@ -1977,6 +1989,8 @@ class FakeSocket:
         elements = args[i:]
         if not elements or len(elements) % 2 != 0:
             raise SimpleError(SYNTAX_ERROR_MSG)
+        if incr and len(elements) != 2:
+            raise SimpleError(ZADD_INCR_LEN_ERROR_MSG)
         # Parse all scores first, before updating
         items = [
             (Float.decode(elements[j]), elements[j + 1])
@@ -1985,6 +1999,12 @@ class FakeSocket:
         old_len = len(zset)
         changed_items = 0
 
+        if incr:
+            item_score, item_name = items[0]
+            if (nx and item_name in zset) or (xx and item_name not in zset):
+                return None
+            return self.zincrby(key, item_score, item_name)
+
         for item_score, item_name in items:
             if (
                 (not nx or item_name not in zset)
@@ -2288,14 +2308,21 @@ class FakeSocket:
         self._server.lastsave = int(time.time())
         return OK
 
+    @command(())
+    def time(self):
+        now_us = round(time.time() * 1000000)
+        now_s = now_us // 1000000
+        now_us %= 1000000
+        return [str(now_s).encode(), str(now_us).encode()]
+
     # Script commands
-    # TODO: script exists, script flush
-    # (script debug and script kill will probably not be supported)
+    # script debug and script kill will probably not be supported
 
     def _convert_redis_arg(self, lua_runtime, value):
-        if isinstance(value, bytes):
+        # Type checks are exact to avoid issues like bool being a subclass of int.
+        if type(value) is bytes:
             return value
-        elif isinstance(value, (int, float)):
+        elif type(value) in {int, float}:
             return '{:.17g}'.format(value).encode()
         else:
             # TODO: add the context
@@ -2447,6 +2474,13 @@ class FakeSocket:
             sha1 = hashlib.sha1(script).hexdigest().encode()
             self._server.script_cache[sha1] = script
             return sha1
+        elif casematch(subcmd, b'exists'):
+            return [int(sha1 in self._server.script_cache) for sha1 in args]
+        elif casematch(subcmd, b'flush'):
+            if len(args) != 0:
+                raise SimpleError(BAD_SUBCOMMAND_MSG.format('SCRIPT'))
+            self._server.script_cache = {}
+            return OK
         else:
             raise SimpleError(BAD_SUBCOMMAND_MSG.format('SCRIPT'))
 
@@ -2546,7 +2580,7 @@ class FakeSelector(BaseSelector):
     def check_can_read(self, timeout):
         if self.sock.responses.qsize():
             return True
-        if timeout <= 0:
+        if timeout is not None and timeout <= 0:
             return False
 
         # A sleep/poll loop is easier to mock out than messing with condition
@@ -2557,7 +2591,7 @@ class FakeSelector(BaseSelector):
                 return True
             time.sleep(0.01)
             now = time.time()
-            if now > start + timeout:
+            if timeout is not None and now > start + timeout:
                 return False
 
     def check_is_ready_for_command(self, timeout):
